/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.samples.camel.example.trace;

import java.io.File;
import java.io.IOException;
import java.util.Properties;

import javax.jms.ConnectionFactory;

import org.apache.activemq.ActiveMQConnectionFactory;
import org.apache.camel.Exchange;
import org.apache.camel.LoggingLevel;
import org.apache.camel.builder.RouteBuilder;
import org.apache.camel.component.jms.JmsComponent;
import org.apache.camel.processor.interceptor.Tracer;
import org.apache.log4j.PropertyConfigurator;
import org.ow2.jonas.samples.camel.service.api.ICamelService;

/**
 * A simple example on how to intercept and trace exchanges while they are on
 * route.
 * 
 * @author Malek chahine
 */
public class ExampleLoader {

    /**
     * Log4J properties file.
     */
    static final String LOG_PROPERTIES_FILE = "/log4j.properties";

    /**
     * The camel service. Injected by the container.
     */
    private ICamelService camelService = null;

    /**
     * The camel context name of the example.
     */
    private String camelContextName = null;

    /**
     * Start the example.
     */
    public void start() throws Exception {

        this.initializeLogger();

        // Start a new context for the application
        this.camelContextName = this.camelService.startNewContext();

        ConnectionFactory connectionFactory = new ActiveMQConnectionFactory("vm://localhost?broker.persistent=false");

        this.camelService.addComponent("test-jms", JmsComponent.jmsComponentAutoAcknowledge(connectionFactory),
            this.camelContextName);

        // Prepare a route to add in the created context
        RouteBuilder builder = new RouteBuilder() {
            @Override
            @SuppressWarnings("deprecation")
            public void configure() throws Exception {
                File camelTest = new File("target/camel-test");
                File source = new File(camelTest, "source");
                File target = new File(camelTest, "target");

                source.mkdirs();
                target.mkdirs();

                // we create a tracer where we want to send TraveEvents to an
                // endpoint

                Tracer tracer = new Tracer();

                // send TraveEvents to an endpoint "direct:traced" where we can
                // do some custom processing and then send it to jms queue
                tracer.setDestinationUri("direct:traced");
                tracer.setLogLevel(LoggingLevel.INFO);

                // use our own formatter instead of the default one
                // MyTraceFormatter formatter = new MyTraceFormatter();
                // tracer.setFormatter(formatter);
                // and we must remeber to add the tracer to Camel

                this.getContext().addInterceptStrategy(tracer);

                this.from(source.toURL().toString()).setHeader(Exchange.FILE_NAME, this.constant("report.txt")).to(
                    target.toURL().toString());

                // configure a route for the traced messages. we process the
                // TraceEventMessage. then we send the traced message to a jms
                // Queue
                this.from("direct:traced").process(new MyTraceMessageProcessor()).to("test-jms:queue:Queue");

            }
        };

        // Add the route in the camel context.
        this.camelService.addRoutes(builder, this.camelContextName);
    }

    /**
     * Set the service instance. Initialized by iPOJO.
     * 
     * @param service the Camel service
     */
    public void setCamelService(final ICamelService service) {
        this.camelService = service;
    }

    /**
     * Stop the context when the service or when the bundle is removed.
     * 
     * @param service the Camel service
     */
    public void unsetCamelService(final ICamelService service) throws Exception {
        try {
            this.camelService.stop(this.camelContextName);
        } finally {
            this.camelContextName = null;
            this.camelService = null;
        }
    }

    /**
     * This method load the Log4J properties file.
     */
    private void initializeLogger() {
        Properties logProperties = new Properties();

        try {
            logProperties.load(this.getClass().getResourceAsStream(LOG_PROPERTIES_FILE));
            PropertyConfigurator.configure(logProperties);
        } catch (IOException e) {
            throw new RuntimeException("Unable to load logging property ");
        }

    }

}
